function [x,its] = conj_grad ( A, b, x, TOL, Nmax )

%CONJ_GRAD    approximate the solution of the linear system Ax = b by applying
%             the conjugate gradient method
%
%     calling sequences:
%             [x,its] = conj_grad ( A, b, x, TOL, Nmax )
%             
%     inputs:
%             A       coefficient matrix for linear system - must be a
%                     square matrix
%             b       right-hand side vector for linear system - must be
%                     a column vector
%             x       column vector containing initial guess for solution of 
%                     linear system
%             TOL     convergence tolerance - applied to Euclidean norm of
%                     residual vector
%             NMax    maximum number of iterations to be performed
%
%     output:
%             x       approximate solution of linear system
%             its     number of iterations
%     NOTE:

n = length ( b );
[r c] = size ( A );
if ( c ~= n ) 
   disp ( 'conj_grad error: matrix dimensions and vector dimension not compatible' )
   return
end;

r = b - A * x;
delta0 = r' * r;
p = r;
its=0;
errore=100;
normb=norm(b);
while its<=Nmax && errore> TOL
    w = A * p;
	alfa = (p' * r) / ( p' * w );
	x = x + alfa * p;
	r = r - alfa * w;
	delta1 = r' * r;
	errore = sqrt(delta1)/normb;
    beta = delta1 / delta0;
	delta0 = delta1;
	p = r + beta * p;
	its=its+1;
end;
	   
